#!/usr/bin/perl
#
#
#		 Naiocast - Yet Another MP3 Streaming Server
#
#		Copyleft (C) 2001-2008 / THEARTOFWEB Software
#
# Naiocast is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Naiocast is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Naiocast; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# Author:
# Giangaetano Vituzzi - THEARTOFWEB Software
#
# You may contact the author of Naiocast by e-mail at:
# naiocast@theartofweb.net
#
# The latest version of Naiocast can be obtained from:
# http://www.naiocast.theartofweb.net
#
# v1.8.4 / ncsstats.cgi / Stable Release
#
#

use strict;
use warnings;
use File::stat;

my @empty;
my @users = <*.user>;
my $users = {};
my %user;

#
# Get active clients
#

my $f = 0;

if (scalar(@users) > 0)
{
	my @active_users; # Active users
	my $sb;           # File::Stat method
	my $lastmod;      # Last mod time
	my $tnew = time;  # Now time
	my $timeout = 5;  # timeout seconds

	foreach my $user (@users)
	{
		chomp($user);
		$sb = stat($user);
		$lastmod = $sb->mtime;
	
		if (($tnew - $lastmod) < $timeout)
		{
			push @active_users, $user;
		}
	}
	
	foreach my $auser (@active_users)
	{
		$f ++;
				
		($user{"port"}, $user{"pid"}, $user{"ip_port"}) = split("_", $auser, 3);
		($user{"raddr"}, undef)                         = split(":", $user{"ip_port"}, 2);
		
		open my $fh1, "<", $auser;
		binmode $fh1;
		read($fh1, $_, 256);
		$user{"ua"} = unpack("a256", $_);
		read($fh1, $_, 256);
		$user{"uptime"} = unpack("a256", $_);
		close($fh1);
		
		$users->{$f}->{REMOTE_ADDR}    = $user{"raddr"};
		$users->{$f}->{USER_AGENT}     = $user{"ua"};
		$users->{$f}->{UP_TIME}        = &_get_uptime($user{"uptime"});
		$users->{$f}->{READER_PID}     = $user{"pid"};
	}
		
} else {

		$users->{$f}->{REMOTE_ADDR}    = "";
		$users->{$f}->{USER_AGENT}     = "";
		$users->{$f}->{UP_TIME}        = "";
		$users->{$f}->{READER_PID}     = "";
}

my ($output, $css);
my $dark = 0;

for my $key ( sort { $users->{$a}->{READER_PID} cmp $users->{$b}->{READER_PID} } keys %$users )
{

	if ($dark == 1) { $css = 'track dark'; $dark = 0 } else { $css = 'track'; $dark = 1 }
	
	$output .= qq|<div class="$css">|;
	$output .= qq|<div class="pid">|.    $users->{$key}->{READER_PID}  ."</div>";
	$output .= qq|<div class="ua">|.     $users->{$key}->{USER_AGENT}  ."</div>";
	$output .= qq|<div class="raddr">|.  $users->{$key}->{REMOTE_ADDR} ."</div>";
	$output .= qq|<div class="port">|.   $users->{$key}->{UP_TIME}     ."</div>";
	$output .= qq|</div>\n|;
}

my $html = qq|
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<title>Naiocast Server &raquo; On Line Listeners ($f)</title>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<meta http-equiv="refresh" content="5" />
<style type="text/css">
<!--
	body { background-color: #fff; padding: 1px; }
	h1 { font-family: "Lucida Grande",Verdana,Arial,sans-serif; font-size: 12px; font-style: italic; margin: 0px; padding: 10px 0px 20px 0px; }
	.trackheaders { background-color: #999; color: #fff; float: left; font-family: Verdana,Arial,sans-serif; font-size: 10px; font-weight: bold; line-height: 16px; margin: 0px 0px 2px 0px; padding: 1px 0px; text-align: center; width: 100%; }
	.track { background-color: #ffd; clear: both; float: left; font-family: Arial,sans-serif; font-size: 11px; line-height: 14px; margin: 0px; padding: 1px 0px; width: 100%; }
	.track div, .trackheaders div { display: block; float: left; padding: 2px 4px; vertical-align: top; }
	.trackheaders a { color: #fff }
	.trackheaders a:visited { color: #fff }
	.dark { background-color: #dfd; }
	.pid   { text-align: center; width: 10%; }
	.ua    { text-align: center; width: 35%; }
	.raddr { text-align: center; width: 25%; }
	.port  { text-align: center; width: 20%; }
	#footer { clear: both; color: #000; font-family: Verdana,Arial,sans-serif; font-size: 11px; font-style: italic; line-height: 14px; padding: 40px 0px; text-align: right; border-top: solid 2px #999; }
//-->
</style>
</head>
<body>
<h1>Naiocast Server &raquo; $f listening</h1>
<div class="trackheaders"><div class="pid">PID</div><div class="ua">USER AGENT</div><div class="raddr">REMOTE ADDR</div><div class="port">UP TIME</div></div>
$output
</body>
</html>|;
print "Content-type: text/html\n\n";
print $html;

##
 # Subroutines
##

sub _get_uptime
{
	my $time     = shift;
	my $tnow     = time;

	my $seconds  = $tnow - $time;

	my $s        = $seconds % 60;
	my $m        = (($seconds - $s) % (60*60))/60;
	my $h        = ($seconds - $m*60 - $s) / (60*60);

	return sprintf( "%1d Hours, %1d Minutes, %1d Seconds", $h, $m, $s );

}