#!/usr/bin/perl
#
#
#		 Naiocast - Yet Another MP3 Streaming Server
#
#		Copyleft (C) 2001-2008 / THEARTOFWEB Software
#
# Naiocast is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Naiocast is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Naiocast; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# Author:
# Giangaetano Vituzzi - THEARTOFWEB Software
#
# You may contact the author of Naiocast by e-mail at:
# naiocast@theartofweb.net
#
# The latest version of Naiocast can be obtained from:
# http://www.naiocast.theartofweb.net
#
# v1.8.4 / nph-play.cgi / Stable Release
#
#

use strict;
use warnings;
use Socket;
use File::stat;
use constant BUFSIZE => 1024 * 4;

select((select(STDOUT), $|=1)[0]);

my ($myself_name, $myself_addr, $myself_host);
my $CRLF = "\015\012";

##
 # install SIGS:
##

$SIG{"PIPE"} = $SIG{"TERM"} = $SIG{"QUIT"} = \&_fine_quit();

##
 # Check if Naiocast Server is broadcasting
##

if (! -e "online.txt")
{
	syswrite STDOUT, "HTTP/1.0 404 Stream not found" . $CRLF;
	syswrite STDOUT, "Server: Apache/1.3.41 (Unix)" . $CRLF;
	syswrite STDOUT, "Connection: close" . $CRLF;
	syswrite STDOUT, "Content-Type: text/html" . $CRLF . $CRLF;
	syswrite STDOUT, '<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN"><html><head><title>404 Entity Not Found</title></head><body><h1>404 Entity Not Found</h1>Naiocast Server is not broadcasting.<br><hr></body></html>';
	exit;
}

##
 # Check Max Listeners
##

my $now_listeners = scalar(&_getlist());
my $max_listeners = &_get_max_listeners();

if ( $now_listeners > $max_listeners )
{
	syswrite STDOUT, "HTTP/1.0 404 Too much listeners" . $CRLF;
	syswrite STDOUT, "Server: Apache/1.3.41 (Unix)" . $CRLF;
	syswrite STDOUT, "Connection: close" . $CRLF;
	syswrite STDOUT, "Content-Type: text/html" . $CRLF . $CRLF;
	syswrite STDOUT, '<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN"><html><head><title>404 Too much listeners</title></head><body><h1>404 Too much listeners</h1>Naiocast Server got max listeners.<br><hr></body></html>';
	exit;
}

##
 # Check Path Info (ENV)
##

if ($ENV{"PATH_INFO"})
{
	($myself_name, $myself_host) = &_get_items_header();
	$myself_addr = $myself_host . $ENV{"SCRIPT_NAME"};

	     if ($ENV{"PATH_INFO"} eq '/listen.pls') {
		&_download_pls();
	} elsif ($ENV{"PATH_INFO"} eq '/listen.xspf') {
		&_download_xspf();
	} elsif ($ENV{"PATH_INFO"} eq '/listen.m3u') {
		&_download_m3u();
	} elsif ($ENV{"PATH_INFO"} eq '/listen.ram') {
		&_download_ram();
	} elsif ($ENV{"PATH_INFO"} eq '/listen.asx') {
		&_download_asx();
	} else {
		syswrite STDOUT, "HTTP/1.0 404 Stream not found" . $CRLF;
		syswrite STDOUT, "Server: Apache/1.3.41 (Unix)" . $CRLF;
		syswrite STDOUT, "Connection: close" . $CRLF;
		syswrite STDOUT, "Content-Type: text/html" . $CRLF . $CRLF;
		syswrite STDOUT, '<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN"><html><head><title>404 Unknown media type</title></head><body><h1>404 Unknown media type</h1>Naiocast Server does not support this media type.<br><hr></body></html>';
	}
	exit;
}

##
 # Get an UDP port number:
##

my ($usock,$uport); ($usock,$uport) = &_udp_setup("127.0.0.1");

##
 # Save client Port numeber and PID:
##

my (%client, $user_id);

$client{"addr"} = $ENV{"REMOTE_ADDR"}     || "null";
$client{"port"} = $ENV{"REMOTE_PORT"}     || "null";
$client{"uage"} = $ENV{"HTTP_USER_AGENT"} || "null";
$user_id        = $uport . '_' . $$ . '_'. $client{"addr"} . ':' . $client{"port"} . '.user';
qx(touch $user_id);

{
	open my $fh1, ">", $user_id or die "$user_id $!\n";
	binmode $fh1;
	print $fh1 pack("a256", $client{"uage"});
	print $fh1 pack("a256", time);
	close $fh1;
}

##
 # Read the stream from the Receiver...
##

my $_header_sent = 0;
my $now          = time;
my $tnew;
my $icym         = &_get_icy_metaint();
my $c = 0;

while( recv($usock, my $buf, BUFSIZE, 0) )
{
	$c += length($buf);

	last unless &_write_to_stdout($buf) > 0;

	$tnew = time;
	qx(touch $user_id) and $now = $tnew if (($tnew - $now) > 2);

	if($c == $icym)
	{
		open my $fh1, "<", "_songtit.txt" or die "$!\n";
		binmode $fh1;
		&_write_to_stdout($_) while sysread $fh1, $_, -s "_songtit.txt";
		close $fh1;
		$c = 0;	
	}
}

close $usock;
&_fine_quit();
exit;

##
 # Subroutines
##

sub _fine_quit
{
	unlink($user_id) if(-e $user_id);
}

sub _udp_setup
{		
	my $HOSTNAME = $_[0];
	my $fh;
	
	socket($fh, PF_INET, SOCK_DGRAM, getprotobyname("udp")) or return "";
	binmode($fh);
	
	my $ipaddr   = inet_aton($HOSTNAME);
	my $portaddr = sockaddr_in(0, $ipaddr);

	bind($fh, $portaddr) or return "";
	
	my ($nport, $naddr);
	($nport, $naddr) = sockaddr_in(getsockname($fh));
	
   	return ($fh,$nport);
}

sub _write_to_stdout
{
	if( $_header_sent == 0 )
	{
		##
		 # Send Naiocast Header (SHOUTcast/Icecast)
		##
		
		{
			open my $fh1, "<", "ncs_header.txt" or die "$!\n";
			binmode $fh1;
			syswrite STDOUT, $_ while sysread $fh1, $_, -s "ncs_header.txt";
			close $fh1;
		}
				
		##
		 # Send ID3v2.3.0 Header
		##
		
		{
			open my $fh1, "<", "id3v2.txt" or die "$!\n";
			binmode $fh1;
			syswrite STDOUT, $_ while sysread $fh1, $_, -s "id3v2.txt";
			close $fh1;
		}
		
		$_header_sent = 1;

		return syswrite STDOUT, substr $_[0], -s "id3v2.txt"
	}
	
	##
	 # Send stream...
	##

	return syswrite STDOUT, $_[0];
}

sub _download_pls
{
	my $output = [
		'[playlist]',
		'NumberOfEntries=1',
		'File1=' . $myself_addr,
		'Length1=-1',
		'Title1=' . $myself_name,
		'Version=2'
	];
	
	my $length = length(join "\n", @$output);
	my $_gmt_date = _get_gmt_date(time);
	
	my $header = [
		'HTTP/1.1 200 OK',
		'Date: ' . $_gmt_date,
		'Server: Apache/1.3.41 (Unix)',
		'Content-Length: ' . $length,
		'Content-Type: audio/x-scpls',
		'Connection: close',
		'',''
	];
	syswrite STDOUT, join $CRLF, @$header;
	syswrite STDOUT, join "\n", @$output;
}

sub _download_xspf
{
	my $output = [
		'<?xml version="1.0" encoding="UTF-8"?>',
		'<playlist version="1" xmlns="http://xspf.org/ns/0/">',
		'<title>' . $myself_name . '</title>',
		'<info>' . $myself_host . '</info>',
		'<trackList>',
		'<track><location>' . $myself_addr . '</location></track>',
		'</trackList>',
		'</playlist>'
	];
	
	my $length = length(join "\n", @$output);
	my $_gmt_date = _get_gmt_date(time);
	
	my $header = [
		'HTTP/1.1 200 OK',
		'Date: ' . $_gmt_date,
		'Server: Apache/1.3.41 (Unix)',
		'Content-Length: ' . $length,
		'Content-Type: application/xspf+xml',
		'Connection: close',
		'',''
	];
	syswrite STDOUT, join $CRLF, @$header;
	syswrite STDOUT, join "\n", @$output;
}

sub _download_m3u
{
	my $length = length($myself_addr);
	my $_gmt_date = _get_gmt_date(time);
	
	my $header = [
		'HTTP/1.1 200 OK',
		'Date: ' . $_gmt_date,
		'Server: Apache/1.3.41 (Unix)',
		'Content-Length: ' . $length,
		'Content-Type: audio/x-mpegurl',
		'Connection: close',
		'',''
	];
	syswrite STDOUT, join $CRLF, @$header;
	syswrite STDOUT, $myself_addr;
}

sub _download_ram
{
	my $length = length($myself_addr);
	my $_gmt_date = _get_gmt_date(time);
	
	my $header = [
		'HTTP/1.1 200 OK',
		'Date: ' . $_gmt_date,
		'Server: Apache/1.3.41 (Unix)',
		'Content-Length: ' . $length,
		'Content-Type: audio/x-pn-realaudio',
		'Connection: close',
		'',''
	];
	syswrite STDOUT, join $CRLF, @$header;
	syswrite STDOUT, $myself_addr;
}

sub _download_asx
{
	my $output = [
		'<ASX version = "3.0">',
		'<Entry>',
		'<REF HREF="' . $myself_addr . '" />',
		'</Entry>',
		'</ASX>'
	];
	
	my $length = length(join "\n", @$output);
	my $_gmt_date = _get_gmt_date(time);
	
	my $header = [
		'HTTP/1.1 200 OK',
		'Date: ' . $_gmt_date,
		'Server: Apache/1.3.41 (Unix)',
		'Content-Length: ' . $length,
		'Content-Type: video/x-ms-asf',
		'Connection: close',
		'',''
	];
	syswrite STDOUT, join $CRLF, @$header;
	syswrite STDOUT, join "\n", @$output;
}

sub _get_gmt_date
{

	# Server-like date: Fri, 10 Oct 2008 14:41:59 GMT
	#
	
	my $tempo = shift;
	my @days   = ('Sun','Mon','Tue','Wed','Thu','Fri','Sat');
	my @months = ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
	my ($sec,$min,$hour,$mday,$mon,$year,$wday) = (localtime($tempo))[0,1,2,3,4,5,6];
	my $time = sprintf("%02d:%02d:%2d",$hour,$min,$sec);
	$year += 1900;
	my $date = "$days[$wday], $mday $months[$mon] $year $time GMT";
}

sub _get_icy_metaint
{
	my ($header_slurp, $header, $icym);
	
	{open my $fh2, "<", "header.txt" or die "header.txt $!\n";
	local $/ = undef;
	$header_slurp = <$fh2>;
	close $fh2;}
	
	($header) = ($header_slurp =~ /<header>(.*?)<\/header>/sg);
	($icym)   = ($header =~ /<icymetaint>(.*?)<\/icymetaint>/sg);
	
	return $icym;
}

sub _get_items_header
{
	my ($header_slurp, $header, $name ,$host);
	
	{open my $fh2, "<", "header.txt" or die "header.txt $!\n";
	local $/ = undef;
	$header_slurp = <$fh2>;
	close $fh2;}
	
	($header) = ($header_slurp =~ /<header>(.*?)<\/header>/sg);
	($name)   = ($header =~ /<name>(.*?)<\/name>/sg);
	($host)   = ($header =~ /<host>(.*?)<\/host>/sg);
	
	return ($name, $host);
}

sub _get_max_listeners
{
	my ($header_slurp, $header, $max);
	
	{open my $fh2, "<", "header.txt" or die "header.txt $!\n";
	local $/ = undef;
	$header_slurp = <$fh2>;
	close $fh2;}
	
	($header) = ($header_slurp =~ /<header>(.*?)<\/header>/sg);
	($max)    = ($header =~ /<max>(.*?)<\/max>/sg);
	
	return $max
}

sub _getlist {

	my @empty;
	my @users = <*.user>;
	my $num = @users;
	
	# Get active clients
	#
	
	if ($num > 0)
	{

		my @active_users; # Active users
		my $sb;           # File::Stat method
		my $lastmod;      # Last mod time
		my $tnew = time;  # Now time
		my $timeout = 10; # timeout seconds
		
		my (@ports, $port);

		foreach my $user (@users)
		{
			chomp($user);
				$sb = stat($user);
					$lastmod = $sb->mtime;
	
			if ( ($tnew - $lastmod) < $timeout )
			{
				push @active_users, $user;
			} else {
				unlink $user;
			}
		}
	
		foreach my $active_user (@active_users)
		{
			($port, undef, undef) = split("_", $active_user, 3);
			push @ports, $port;
		}
		
		return @ports;
		
	} else {

		return @empty;

	}

}